/*****************************************************************************
 *
 * cdflush.c - roto-root a port and attempt to free it when wedged badly
 *
 * Copyright 1999-2000 Digi International (www.digi.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/sysmacros.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include "ctio.h"
#include "cdutils.h"

struct stat s;

#define	SZ	256
char buf[SZ];

#define	TRUE	1
#define	FALSE	0


/* globals */
char *progname;

/* protos */
void usage(void);

/*
 *	main
 */
int main(int argc, char **argv) {
  
	int index;
	char *devname;
	int gfd, minordev;
	int status, code;
  
	progname = argv[0];
  
	/* make sure at least one port spec'd */
	if (argc < 2) 
		usage();
	
	/* make sure they're root */
	if (geteuid() != 0) {
		printf("You must be root to use this utility.\n");
		exit(1);
	}
  
	/* try to open the global diag node */
	gfd = open(DEV_DIAG,O_RDWR);
	if (gfd < 0) {
		perror(DEV_DIAG);
		printf("Couldn't open global diagnostics node.\n");
		exit(1);
	}
  
	/* check all spec'd terminals */
	for (index = 1; index < argc; index++) {
		devname = argv[index];
    
		if (stat(devname,&s) < 0) {
			perror(NULL);
			printf("%s: couldn't stat '%s'\n",progname,devname);
			continue;
		}
		if (!(S_ISCHR(s.st_mode))) {
			printf("%s: '%s' not a character device\n",progname,devname);
			continue;
		}
    
		minordev = minor(s.st_rdev);
		status = ioctl(gfd, STSFLUSH, minordev);
    
		if (status < 0) {
			code = errno;
			switch (code) {
			case EPERM:
				printf("Permission denied.\n");
				break;
			case ENODEV:
				printf("Unit was not found.\n");
				break;
			case EINVAL:
				printf("Port number is out of range.\n");
				break;
			case ENOENT:
				printf("Line has never been accessed.\n");
				break;
			case ENOTTY:
				printf("Line is not currently enabled.\n");
				break;
			case EIO:
				printf("Queues not active, only firmware was flushed.\n");
				break;
			case ENXIO:
				printf("SIGHUP cleared streams queues.\n");
				break;
			default:
				printf("Unrecognized return value %d - %s\n",
					   code, strerror(code));
				break;
			}
		}
		else {
			printf("Port '%s' flushed.\n", argv[index]);
		}
	}
	close(gfd);
	exit(0);
}



/*
 *	usage
 */
void usage(void) 
{
	printf("Usage: cdflush [devname(s)]\n");
	exit(1);
}










