/*
 * dtrcels.c - display trace buffer out of ELS debug driver.
 *
 *****************************************************************************
 *
 * Copyright 1999-2000 Digi International (www.digi.com)
 *    Zhong Deng <Zhong_Deng@digi.com>
 *    Jeff Randall <Jeff_Randall@digi.com>
 *    Bob Rubendunst <Robert_Rubendunst@digi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/sysmacros.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include "ctio.h"
#include "cdutils.h"


unsigned char buf[0x10000];

/* protos */
void usage(void);
void unwrap(int trcsz);

/*
 * main()
 */
int main(int argc, char **argv)
{
	int cmd;
	int fd, stat, minor_dev, error;

  
	if (argc == 2 && !strcmp(argv[1], "-c"))
		cmd = STSCLRTRCBUF;
	else
		cmd = STSGETTRCBUF;
	
	minor_dev = 0;
	
	/* open the port */
	fd = open(DEV_DIAG, O_RDWR);
	if (fd < 0) {
		printf("Couldn't open diagnostic node %s: %s\n",
		       DEV_DIAG, strerror(errno));
		exit(1);
	}

	/* prepare the actual ioctl */
	stat = ioctl(fd, cmd, buf);
	if (stat < 0) {
		error = errno;
		switch(error) {
		case ENOENT:
			fprintf(stderr,
				"Driver not configured for (that type of) "
				"tracing.\n");
			exit(1);
		case EINVAL:
			fprintf(stderr,
				"Specified device was not found.\n");
			exit(1);
		case ENODEV:
			fprintf(stderr,
				"Line has never been active.  No trace "
				"data.\n");
			exit(1);
		default:
			perror("STSTRCBUF");
			fprintf(stderr,"Unknown error fetching trace "
				"buffer.\n");
			exit(1);
		}
	}
  
	/* close the diag node */
	close(fd);
  
	/* now unwrap the buffer and display */
	unwrap(sizeof(buf));
	
	printf("\nEND\n");
	
	/* we're done */
	exit(0);
}

/*
 * unwrap()
 *
 * Look for tilde in the buffer indicating the wrap point.
 * Display from wrap point to end, and then from start to
 * wrap point.
 *
 * If no tilde found, just show whole buffer in order.
 */
void unwrap(int trcsz)
{
	int i, wrap, found_data;
  
	/* find tilde */
	for(i = found_data = 0; i < trcsz; i++) {
		found_data |= buf[i];
		if(buf[i] == '~')
			break;
	}
  
	/* make sure we found data and a tilde */
	if(!found_data) {
		printf("No data in trace buffer.\n");
		return;
	}
	else if(i == trcsz) {
		printf("No wrap point found!\n");
		wrap = 0;
	}
	else 
		wrap = i;
	
	/* display oldest chunk */
	for(i = wrap + 1; i < trcsz; i++)
		if(buf[i]) putchar(buf[i]);
	
	/* display newer chunk */
	for(i = 0; i < wrap; i++)
		if(buf[i]) putchar(buf[i]);
}




