/*
 *  Internal Header file for the Digi International EtherLite(R) Terminal
 *  Server driver for Linux
 *
 *****************************************************************************
 *
 * Copyright 1999-2000 Digi International (www.digi.com)
 *    Zhong Deng <Zhong_Deng@digi.com>
 *    Jeff Randall <Jeff_Randall@digi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************
 */

/* use Printf for permanent prints, normal lower case for temp debug ones */
#define	Printf printk

/*
 *	Universal TRUE and FALSE definitions if not defined by the system.
 */
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

/* for debugging you can #define LOCAL */
#define	LOCAL static
#define	GLOBAL


/*
 * Tracer stuff for debug
 *
 * Here's what the defines do:
 *	TRACER globally enables/disables tracing
 *
 */
/* define appropriate macros and trace sizes */
#if defined(TRACER)
#	define	trc(x)	tracer x 
#	define	ntrc(x)	tracer x
#	define	ttrc(x)	
#else
#	define	trc(x)
#	define	ntrc(x)	
#	define	ttrc(x)	
#endif


/* ELS stuff */
#define MAX_ELS_DEVS	7	/* max # of EtherLite units per module */
#define MAX_ELS_LINES	32	/* max # of lines per EtherLite product */
#define NR_PORTS	256

#define MAX_MODULE	9	/* 1 module can support up to 7 units */



/* 
 * Minor device number scheme for ELS (use 8 bits tentatively)
 *
 *   bits 0-4        port number  (0-31)  if bits 5-6 are not 11
 *   bits 5-7        device type
 *                      000 -- 110: port on unit 0 through 6
 *                      111: unit node where bits 0-2 determine unit number
 *			     (from 0 to 6)
 *   special case    0xff for diag node
 */

#define DEV2UNIT(dev)	(((dev) >> (ELS_UNIT(dev) ? 0 : 5)) & 0x07)
#define DEV2LINE(dev)	((dev) & 0x1f)
#define ELS_UNIT(dev)	(((dev) & 0xe0) == 0xe0)
#define ELS_LINE(dev)	(((dev) & 0xe0) != 0xe0)

/*
 * GLOBAL_MINOR
 *
 * Special device # recognized for diag and util entry point into driver.
 */
#define	ELS_GLOBAL_MINOR	0xff



/* some macros for loading low/high bytes */
#define load_lo(var)  ((var & 0x00ff))
#define load_hi(var) (((var & 0xff00)) >> 8)

/* driver defs */
#define	MAX_PKT			4096
#define	CQSIZE			4096
#define	FASCMDSZ		8
#define	FASHISTSZ		32
#define	CTF_TIMER_INTERVAL	10
#define	CTF_OFFLINE_HOLDOFF	20

/* macro for deciding whether it's a local line */
#define	NONLOCAL(lp) \
	(((lp)->tty->driver.subtype == SERIAL_TYPE_CALLOUT) ||	\
	 (((lp)->flags & ASYNC_CALLOUT_ACTIVE) &&		\
	  ((lp)->normaltermios.c_cflag & CLOCAL)) ||		\
	 !(((lp)->flags & ASYNC_CALLOUT_ACTIVE) &&		\
	   ((lp)->tty->termios->c_cflag & CLOCAL)))

/* unit level state defs */
#define	UNIT_RESET		0x0001
#define	UNIT_INITING		0x0002
#define	UNIT_INITD		0x0004
#define	UNIT_OFFLINE		0x0020
#define UNIT_UTILITY		0x0040
#define UNIT_IDENT		0x0080
#define UNIT_DIRECT		0x0100
#define UNIT_LOCK		0x0200

/* line level state defs */
#define LINE_ENABLED		0x00000001
#define LINE_DISABLING		0x00000002
#define LINE_OFFLINE		0x00000004
#define LINE_LOOPBACKON		0x00000008
#define LINE_BREAK		0x00000010
#define LINE_WAIT_OUT		0x00000020
#define LINE_SET_MODEM		0x00000040
#define LINE_SET_PARAMS		0x00000080
#define LINE_GET_MODEM		0x00000200
#define LINE_PRINTER		0x00000400
#define LINE_ISTOPPED		0x00001000
#define LINE_IPENDING		0x00002000
#define LINE_OBUSY		0x00008000
#define LINE_OSTOPPED		0x00010000
#define LINE_WAIT_TIMER         0x00040000
#define LINE_WAIT_IFLUSH        0x00080000
#define LINE_RESET              0x00100000

/* open close bitops */
#define OCSEMA		0

/* firmware feature flags for 'fw_features' fields of unit struct */
#define FW_RSL          0x0001	/* supports Resilient SCSI layer */
#define FW_RESET	0x0002	/* supports FAS_RESET command */
#define FW_WAIT_SEND	0x0004	/* supports FAS_WAIT_SEND command */
#define FW_MDM_PERSIST	0x0008	/* modem sigs can persist after disable */
#define FW_NO_SWALLOW	0x0010	/* special flavor of XON/XOFF flow ctrl */


#if !defined(KERNEL_VERSION)
# define KERNEL_VERSION(a,b,c) (((((a)<<8)+(b))<<8)+(c))
#endif

/*
 * Locking stuff 
 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
#define ELS_LOCK(lock,flags) spin_lock_irqsave(&(lock), flags)
#define ELS_UNLOCK(lock,flags) spin_unlock_irqrestore(&(lock), flags)
#define LINE_LOCK(lock,flags) spin_lock_irqsave(&(lock), flags)
#define LINE_UNLOCK(lock,flags) spin_unlock_irqrestore(&(lock), flags)
#else
#define ELS_LOCK(lock,flags) {save_flags(flags);cli();}
#define ELS_UNLOCKX(lock, flags)	restore_flags(flags)
#define LINE_LOCK(lock,flags) {save_flags(flags);cli();}
#define LINE_UNLOCKX(lock, flags)	restore_flags(flags)
#endif

#define min(x, y) ((x) < (y) ? (x) : (y))
#define max(x, y) ((x) > (y) ? (x) : (y))

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
#define COPY_FROM_USER(DST,SRC,LEN)	__copy_from_user(DST,SRC,LEN)
#define COPY_TO_USER(DST,SRC,LEN)	__copy_to_user(DST,SRC,LEN)
#define GET_USER(A1,A2)			__get_user(A1,(unsigned int *)A2)
#define PUT_USER(A1,A2)			__put_user(A1,(unsigned long *)A2)
#else
#define COPY_FROM_USER(DST,SRC,LEN)	memcpy_fromfs(DST,SRC,LEN)
#define COPY_TO_USER(DST,SRC,LEN)	memcpy_tofs(DST,SRC,LEN)
#define GET_USER(A1,A2)			A1 = get_fs_long((unsigned long *) A2)
#define PUT_USER(A1,A2)			put_fs_long(A1, (unsigned long *) A2)
#endif


/* data for circular q */
typedef struct circle_q {
	int	cnt;	/* # chars in queue */
	u_char *fp;	/* filling ptr */
	u_char *ep;	/* emptying ptr */
	u_char *end;
	u_char  d[CQSIZE];	/* data storage */
} circle_q;

/*
 * ct_pkt format for muxing data to async lines
 */
typedef struct ct_pkt {
	struct ct_pkt	*link;
	u_char		*dp;
	int 		 space_remaining;
	u_char		 d[MAX_PKT];
} ct_pkt;

#define CT_MAGIC	 7221969

/* line structure - per port data */
typedef
struct ct_line {
	 /*
	  * ---->DO NOT MOVE ct_magic from head of structure as
	  * cddiag uses this for devnum transport
	  */
	int	ct_magic;		/* ct_magic used for sanity checks. */
	int	ocsema;			/* open close semaphore */
	int	line_state;		/* utility state flags */
	int	flags;			/* Linux ASYNC flags */
	int	curr_size_fassend;	/* current max packet based on rtt */
	int	print_max;		/* max packet for printer ports */
	int	nonloc_timeout; 	/* delay secs - non-local disables */
	int	local_timeout;		/* delay secs - local disables */
	int	local_waitout;		/* local output wait watchdog (secs) */
	int	nonloc_waitout; 	/* non-local output watchdog (secs) */
	int	cdbaud_ext;             /* baud rate extension from cdstty */
	int	cdbaud_delta;		/* flag the baudext changed */
	int	int_obaud;		/* baud rate as an integer e.g.57600 */
	u_int	mdmsigs;		/* current state of modem sigs */
	u_int	old_mdmsigs;		/* previous state of modem sigs */
	struct ct_unit *up;		/* ptr to unit assoc'd w/ this line */
	int	offline_time;		/* time unit went AWOL */
	u_int	error_level;		/* error printig level for this line */
	int	refcount;
	int	openwaitcnt;
	int	wait_mask;
	int	next_hist;
	u_char	fas_hist[FASCMDSZ * FASHISTSZ];
	u_char *fas_histptr;		/* pointer into FAS history buf */
	struct tty_struct *tty; 	/* associated tty */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
	wait_queue_head_t open_wait;
        wait_queue_head_t close_wait;
        wait_queue_head_t mdmsig_wait;
#else
	struct wait_queue *open_wait;
	struct wait_queue *close_wait;
	struct wait_queue *mdmsig_wait;
#endif
	struct termios normaltermios;
	struct termios callouttermios;
	struct serial_icounter_struct icount;
	long	session;
	long	pgrp;
	int	closing_wait;
	int	close_delay;

	circle_q	xmit_q;
	req_set_params	last_parms;	/* copy of last set_params sent out */
	req_flow_ctrl	last_flow;	/* copy of last flow_ctrl sent out */

	u_char	enab_disab_stat;	/* error return for enab/disab */
	u_char	port;			/* line # at unit */
	u_char	monitor_lines;		/* status to monitor via FAS_STAT_CHG*/
	u_char	xchar;			/* high-priority char */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	spinlock_t	 linelock;
#endif
} ct_line;


/* unit structure - one per STS SCSI device */
typedef
struct ct_unit {
	int	unit_flags;		/* utility state flags */
	int	num_lines;		/* max lines at this unit */
	int	pun;			/* pun # (2nd digit of els # */
	int	unum;			/* unit number (displayed %2.2x) */
	struct ctunitinfo *uinfo;	/* ptr to product-specific record */
	char	fw_ver[5];		/* handy null-terminated copy */
	ct_line **ct_lines;		/* array of lines */
	int	max_opkt;		/* max ct_pkt size for this unit */
	int	max_size_fassend;	/* max packet this line can output */
	int	fw_features;		/* flags for FW version ID */
	int	unit_offline_time;	/* time unit went AWOL */
	ct_pkt *pkt_head;		/* head of outgoing queue */
	ct_pkt *pkt_tail;		/* tail of outgoing queue */
	ct_pkt *recv_ctpkt;		/* permanent ct_pkt we recv into */
	u_char	initd;			/* flag to indicate initialized */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
        wait_queue_head_t read_wait;
        wait_queue_head_t write_wait;
#else
	struct wait_queue *read_wait;
	struct wait_queue *write_wait;
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	spinlock_t	 faslock;
#endif

/*
 * EtherLite stuff
 */
  unsigned	netaddr;	/* internet address (dot decimal) */
  circle_q	netdata_out;	/* data sent out to inet */
  circle_q	netdata_in;	/* data received from inet */
  int		active_time;	/* last time active f/w msg recv'd */
  int		avg_rtt;	/* average round trip time */
  int		avg_rtt_dev;	/* deviation of avg rtt (+/-) */
  int		poll_time;	/* last unit poll time */
} ct_unit;





