/*
 * Copyright (C) 2000,2001 Kees Cook
 * cook@cpoint.net, http://outflux.net/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * http://www.gnu.org/copyleft/gpl.html
 *
 */
#ifndef STS_H
#define STS_H
#include "scsi_tools.h"

/* shamelessly stolen from EXT2 headers... */
/*
 * Define LOG_DEBUG to produce debug logs to stderr
 */
#undef STDERR_DEBUG
/*
 * Define STS_DEBUG to produce debug messages
 */
#undef	STS_DEBUG
/*
 * Define SCSI_DEBUG to produce debug messages for scsi calls (needs STS_DEBUG)
 */
#undef SCSI_DEBUG
/*
 * Define SELECT_DEBUG to produce debug messages during select loop
 * (requires STDERR_DEBUG)
 */
#undef SELECT_DEBUG


/*
 * Debug code
 *                                 printf ("sts DEBUG (%s, %d): %s:", \
 *                                          __FILE__, __LINE__, __FUNCTION__); \
 */
#ifdef STS_DEBUG
# ifndef STDERR_DEBUG
#  define STDERR_DEBUG
# endif
#       define sts_debug(f, a...) { \
                                  printf (f, ## a); \
                                  }
#else
#       define sts_debug(f, a...)      /**/
#endif


#if 0
#include "fas.h"
#endif

/* for typing convenience */
#define SG_LEN          sizeof(struct sg_header)
#define MAX_PACKET      4096
#define FAS_PKT_MAX	2048 /* not true, perhaps? */
#define FAS_SCSI	1 /* FAS block from SCSI bus */
#define FAS_ELS		0 /* FAS block from ELS driver */
#ifndef PKT_END
#define PKT_END		0x64 /* FAS end marker for SCSI */
#define PKT_ERR		0xFF /* my marker to show I'm reading too far */

#define FAS_CMD_LEN     8    /* bytes in a FAS_CMD header */

#define FAS_GLOBAL      0x00 /* Sets global firmware options for all lines */
#define FAS_ENABLE      0x01 /* Enables specific async line,
                                initializing to defaults; must be
                                issued before any other command to a line */
#define FAS_DISABLE     0x02 /* Disables specific async line, turning
                                off interrupts, etc. */
#define FAS_SEND        0x03 /* Sends data for output */

#define FAS_RECV        0x04 /* Receives input data */
#define FAS_IN_TIMERS   0x05 /* Sets custom block transfer parameters */
#define FAS_OUTPUT_CTL  0x06 /* Flushes, suspends, or resumes output at
                                a line */
#define FAS_INPUT_CTL   0x07 /* Flushes, suspends, or resumes input at
                                a line */
#define FAS_SET_MODEM   0x08 /* Controls modem output signals */
#define FAS_STAT_CHG    0x09 /* Monitors modem input signals */
#define FAS_SEND_BRK    0x0A /* Issues break indication */
#define FAS_SET_PARAMS  0x0B /* Sets line parameters (e.g., baud rate,
                                bits per character, stop bits) */
#define FAS_FLOW_CTL    0x0C /* Sets firmware flow control parameters */
#define FAS_RESERVE     0x0D /* Requests private use of a line */
#define FAS_RELEASE     0x0E /* Releases private use of a line */
/* from EL fas.h ... */
#define FAS_RESET                       15
#define FAS_UNIT_STATS          16
#define FAS_WAIT_SEND           17
#define FAS_LINE_STATE          18
#define FAS_LINE_INQUIRY        19
#define FAS_LPT_TEST            20
#define FAS_UNIT_POLL           21
#define FAS_UNIT_UNLOCK         22
#define FAS_UNIT_INQUIRY        23
#define FAS_EXT_PARAMS          24
#define FAS_UNIT_RESET          25
#define FAS_CONFIG_LINK         50
#define FAS_MUX_STATS           51
#define FAS_CMD_MAX		(FAS_MUX_STATS+1)
#endif


/* scsi vendor info offsets */
#define EXTRA_OFF       4       /* where the extra len is */
#define VENDOR_START    8       /* Offset in reply data to vendor name */
#define VENDOR_LEN      8       /* length of vendor string */
#define PRODUCT_LEN     16      /* length of product string */
#define REV_LEN         4       /* length of rev string */
#define EXTRA_LEN       20      /* length of the extra string */

/* cmd list info */
typedef struct cmd_list_t {
        unsigned char * data;
        int size;
        struct cmd_list_t * prev;
        struct cmd_list_t * next;
} cmd_list;

/* sts structure */
typedef struct sts_state_t {
        int host, chan, id;      /* SCSI identifiers */
        char vendor[VENDOR_LEN + 1];   /* SCSI manufacturer string */
        char product[PRODUCT_LEN + 1]; /* SCSI model string */
        char rev[REV_LEN + 1];         /* SCSI revision */
        char sendextra[EXTRA_LEN + 1]; /* SCSI vendor extras */
        char recvextra[EXTRA_LEN + 1]; /* SCSI vendor extras */
        scsi_info * senddev;	/* SCSI device for Send LUN */
	scsi_info * recvdev;    /* SCSI device for Recv LUN */
        cmd_list * head;        /* head of cmd list */
        cmd_list * tail;        /* tail of cmd list */
        int writable;           /* can I post an STS write cmd? */
        int readable;           /* can I post an STS read cmd? */
	int flushed;		/* msg recv are clear */
        int globaled;           /* did we init the sts? */
	int portsensed;		/* did we finish sensing the number of ports?*/
	int shutdown;		/* shutdown requested */
} sts_state;

int recv_fas_block(sts_state * sts);
int find_fas_block_size(sts_state * sts, char * block, int maxsize,
  char * fascmds, int bufsize, int fromscsi);

int sts_write(sts_state * sts, unsigned char * data_buf, int len);
int sts_read(sts_state * sts, char * data_buf, int n);
sts_state * sts_shutdown(sts_state * sts);

#endif
